/*
--------------- LCD L020b 176x132 display Soft + hard SPI driver -------------
by Master Mushi
Designed from simple driver by Christian Kranz
Uses Software SPI code snipets by Maxim Integrated Products

Short hint:
lcd_init() - RUN THIS FUNCTION FIRST to get your LCD active and prepared!
ls020_power_OFF() - run this feature to pot the screen in HALT

Function Help:

IO functions:
SoftSPI_init() - Software SPI interface initialization
HardSPI_init() - hardware SPI interface initialization
SPI_write(byte_to_send, mode) - software SPI data send function. Adopted to work with LCD 
mode here means Command (set to 1) or Data (any other value) mode
lcd_wrcmd16(DATA) - command output in a single value
lcd_wrdat16(DATA) - data output in a single value

Graphic functions
lcd_clear() - clears the screen with the WHITE color
lcd_fill(COLOR) - fills LCD with the specified color
lcd_putpixel(x, y) - puts a pixel at the screen with the text color
lcd_cputpixel(x, y, color) - puts a pixel at the screen with the specified color
lcd_fillrect(x1, y1, x2, y2, color) - fills a rectangle with color
lcd_line (x1, y1, x2, y2, color) - draws a line

Text functions:
lcd_gotoxy(X, Y) - moves a TEXT cursor to a specified position
lcd_putcharxy(X, Y, CHAR) - outputs a single symbol at the specified position
lcd_putchar(CHAR) - outputs a single symbol at the current cursor position and moves cursor rightwards
lcd_puts() - outputs string

Additional functions;
lcd_color16(color24) - translates 24bit(8R 8G 8B) color to 16bit(5R 6G 5B) returns unsigned INT

GRAPHIC COORDINATES

Pleased remember that screen starts from the top left corner in PORTRAIT mode!!
          
0--------132 X 132---------------176
|          |   |                   |
|          |   |                   |
|          |   |                   |
|          |   |                   |
|          |   0-----------------176
|          |
|          |
|          |
|          |
176------132
Y

Coordinates are rotation dependend. 
If lcd_rot is set to one then text coords looks like

TEXT COORDINATES

TEXT goes from the Right Left to Down Right
like in any noirmal screen.
number of chars per line depends from the font settings.

0----------n   0-------------------n 
|          |   |                   |
|          |   |                   |
|          |   |                   |
|          |   |                   |
|          |   m------------------nm
|          |
|          |
|          |
|          |
m---------nm
Y

If cursor reaches max coordinates (n, m) then it moves to )0, 0) 

*/


#ifndef _DELAY_INCLUDED_
#include <delay.h>
#endif


#ifndef _STRING_INCLUDED_
#include <string.h>
#endif


#ifndef _STDIO_INCLUDED_
#include <stdio.h> 
#endif

#ifndef _SPI_INCLUDED_
#include <spi.h>
#endif


#ifndef _L020LCD_
#define _L020LCD_
#endif


//Set your prefirable SPI type here
//#define SOFT_SPI
#define HARD_SPI  

//Set your LCD/SPI port pins here

#ifndef LCD_RS
#define LCD_RS	   PORTB.5
#endif

#ifndef DLCD_RS
#define DLCD_RS	   DDRB.5
#endif
//------------------------

#ifndef LCD_RESET
#define LCD_RESET  PORTB.6
#endif

#ifndef DLCD_RESET
#define DLCD_RESET  DDRB.6
#endif
//------------------------

#ifndef LCD_CS
#define LCD_CS	   PORTB.0
#endif

#ifndef DLCD_CS
#define DLCD_CS	   DDRB.0
#endif
//------------------------

#ifndef LCD_SCK 
#define LCD_SCK    PORTB.5
#endif

#ifndef DLCD_SCK 
#define DLCD_SCK    DDRB.5
#endif
//------------------------

#ifndef LCD_MOSI
#define LCD_MOSI   PORTB.3
#endif

#ifndef DLCD_MOSI
#define DLCD_MOSI   DDRB.3
#endif
//------------------------
 
#ifndef LCD_MISO
#define LCD_MISO   PORTB.4
#endif

#ifndef DLCD_MISO
#define DLCD_MISO   DDRB.4
#endif
//------------------------

#ifndef LCD_SS 
#define LCD_SS     PORTB.2
#endif

#ifndef DLCD_SS 
#define DLCD_SS     DDRB.2
#endif
//------------------------



//Named Color palette for the ease of use 
#define RED 0xF800      //#FF0000
#define GREEN 0x0400    //#008000
#define BLUE 0x001F     //#0000FF
#define YELLOW 0xFFE0   //#FFFF00
#define MAGENTA 0xF81F  //#FF00FF
#define CYAN 0x07FF     //#00FFFF
#define LIME 0x07E0     //#00FF00
#define VIOLET 0x3807   //#8A2BE2
#define MAROON 0x3800   //#800000
#define NAVY 0x0007     //#000080
#define OLIVE 0x38E0,   //#808000
#define DARKGREEN 0x01E0//#006400
#define WHITE 0xFFFF    //#FFFFFF
#define GREY 0x79EF     //#7F7F7F
#define DARKGREY 0x38E7 //#404040
#define BLACK 0x0000    //#000000


#define DISP_W 132
#define DISP_H 176

#define ASX 0x1200
#define ASY 0x1300
#define APX 0x1400
#define AEX 0x1500
#define AEY 0x1600
#define APY 0x1700


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//!!                                  !!
//!!        Screen Setting!           !!
//!!     Please use these vars        !!
//!!     To tune up your screen       !!
//!!                                  !!
//!!                                  !!
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

unsigned int backcolor=0xFFFF;    // color of background
unsigned int textcolor=0x0000;    // Text characters color

#define LCD_ROT
//#define LCD_ROT_NOT
//unsigned char lcd_rot=1;          // Screen Orientattion Vertical 0 or Horizontal 1


#ifndef _FONT
#define CHAR_H 14
#define CHAR_W 8
#define TEXT_COL 14
#define TEXT_ROW 9
// ascii table, starting with character blank (32)
// size is 8x14
flash char ascii_tab[96][14]={
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // space (32)
{ 0x00, 0x00, 0x00, 0x18, 0x3c, 0x3c, 0x3c, 0x18, 0x18, 0x00, 0x18, 0x18, 0x00, 0x00}, //!
{ 0x00, 0x66, 0x66, 0x66, 0x24, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, //"
{ 0x00, 0x00, 0x00, 0x6c, 0x6c, 0xfe, 0x6c, 0x6c, 0x6c, 0xfe, 0x6c, 0x6c, 0x00, 0x00}, //#
{ 0x00, 0x18, 0x18, 0x7c, 0xc6, 0xc2, 0xc0, 0x7c, 0x06, 0x86, 0xc6, 0x7c, 0x18, 0x18}, //$
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0xc2, 0xc6, 0x0c, 0x18, 0x30, 0x66, 0xc6, 0x00, 0x00}, // %
{ 0x00, 0x00, 0x00, 0x38, 0x6c, 0x6c, 0x38, 0x76, 0xdc, 0xcc, 0xcc, 0x76, 0x00, 0x00}, // &
{ 0x00, 0x18, 0x18, 0x18, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // '
{ 0x00, 0x00, 0x00, 0x0c, 0x18, 0x30, 0x30, 0x30, 0x30, 0x30, 0x18, 0x0c, 0x00, 0x00}, // (
{ 0x00, 0x00, 0x00, 0x30, 0x18, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x18, 0x30, 0x00, 0x00}, // )
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x66, 0x3c, 0xff, 0x3c, 0x66, 0x00, 0x00, 0x00, 0x00}, // *
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x7e, 0x18, 0x18, 0x00, 0x00, 0x00, 0x00}, // +
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x18, 0x30, 0x00}, // 
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // -
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00}, // .
{ 0x00, 0x00, 0x00, 0x02, 0x06, 0x0c, 0x18, 0x30, 0x60, 0xc0, 0x80, 0x00, 0x00, 0x00}, // /
{ 0x00, 0x00, 0x00, 0x38, 0x6c, 0xc6, 0xc6, 0xd6, 0xc6, 0xc6, 0x6c, 0x38, 0x00, 0x00}, // 0 (48-32)
{ 0x00, 0x00, 0x00, 0x18, 0x38, 0x78, 0x18, 0x18, 0x18, 0x18, 0x18, 0x7e, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x7c, 0xc6, 0x06, 0x0c, 0x18, 0x30, 0x60, 0xc6, 0xfe, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x7c, 0xc6, 0x06, 0x06, 0x3c, 0x06, 0x06, 0xc6, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x0c, 0x1c, 0x3c, 0x6c, 0xcc, 0xfe, 0x0c, 0x0c, 0x1e, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xfe, 0xc0, 0xc0, 0xc0, 0xfc, 0x06, 0x06, 0xc6, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x38, 0x60, 0xc0, 0xc0, 0xfc, 0xc6, 0xc6, 0xc6, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xfe, 0xc6, 0x06, 0x0c, 0x18, 0x30, 0x30, 0x30, 0x30, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x7c, 0xc6, 0xc6, 0xc6, 0x7c, 0xc6, 0xc6, 0xc6, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x7c, 0xc6, 0xc6, 0xc6, 0x7e, 0x06, 0x06, 0x0c, 0x78, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00, 0x00, 0x18, 0x18, 0x30, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x0c, 0x18, 0x30, 0x60, 0xc0, 0x60, 0x30, 0x18, 0x0c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7e, 0x00, 0x00, 0x7e, 0x00, 0x00, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x60, 0x30, 0x18, 0x0c, 0x06, 0x0c, 0x18, 0x30, 0x60, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x7c, 0xc6, 0xc6, 0x0c, 0x18, 0x18, 0x00, 0x18, 0x18, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x7c, 0xc6, 0xc6, 0xde, 0xde, 0xde, 0xdc, 0xc0, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x10, 0x38, 0x6c, 0xc6, 0xc6, 0xfe, 0xc6, 0xc6, 0xc6, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xfc, 0x66, 0x66, 0x66, 0x7c, 0x66, 0x66, 0x66, 0xfc, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x3c, 0x66, 0xc2, 0xc0, 0xc0, 0xc0, 0xc2, 0x66, 0x3c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xf8, 0x6c, 0x66, 0x66, 0x66, 0x66, 0x66, 0x6c, 0xf8, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xfe, 0x66, 0x62, 0x68, 0x78, 0x68, 0x62, 0x66, 0xfe, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xfe, 0x66, 0x62, 0x68, 0x78, 0x68, 0x60, 0x60, 0xf0, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x3c, 0x66, 0xc2, 0xc0, 0xc0, 0xde, 0xc6, 0x66, 0x3a, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xc6, 0xc6, 0xc6, 0xc6, 0xfe, 0xc6, 0xc6, 0xc6, 0xc6, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x3c, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x3c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x1e, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0xcc, 0xcc, 0x78, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xe6, 0x66, 0x6c, 0x6c, 0x78, 0x6c, 0x6c, 0x66, 0xe6, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xf0, 0x60, 0x60, 0x60, 0x60, 0x60, 0x62, 0x66, 0xfe, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xc6, 0xee, 0xfe, 0xd6, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xc6, 0xe6, 0xf6, 0xfe, 0xde, 0xce, 0xc6, 0xc6, 0xc6, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x7c, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xfc, 0x66, 0x66, 0x66, 0x7c, 0x60, 0x60, 0x60, 0xf0, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x7c, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0xd6, 0xde, 0x7c, 0x0e, 0x00},
{ 0x00, 0x00, 0x00, 0xfc, 0x66, 0x66, 0x66, 0x7c, 0x6c, 0x66, 0x66, 0xe6, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x7c, 0xc6, 0xc6, 0x60, 0x38, 0x0c, 0xc6, 0xc6, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x7e, 0x7e, 0x5a, 0x18, 0x18, 0x18, 0x18, 0x18, 0x3c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0xc6, 0x6c, 0x38, 0x10, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xc6, 0xc6, 0xc6, 0xc6, 0xd6, 0xd6, 0xfe, 0x6c, 0x6c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xc6, 0xc6, 0xc6, 0x7c, 0x38, 0x7c, 0xc6, 0xc6, 0xc6, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x66, 0x66, 0x66, 0x66, 0x3c, 0x18, 0x18, 0x18, 0x3c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xfe, 0xc6, 0x8c, 0x18, 0x30, 0x60, 0xc2, 0xc6, 0xfe, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x3c, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x3c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x70, 0x38, 0x1c, 0x0e, 0x06, 0x02, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x3c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x3c, 0x00, 0x00},
{ 0x10, 0x38, 0x6c, 0xc6, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff},
{ 0x00, 0x30, 0x18, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x78, 0x0c, 0x7c, 0xcc, 0xcc, 0x76, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0xe0, 0x60, 0x60, 0x78, 0x6c, 0x66, 0x66, 0x66, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7c, 0xc6, 0xc0, 0xc0, 0xc6, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x1c, 0x0c, 0x0c, 0x3c, 0x6c, 0xcc, 0xcc, 0xcc, 0x76, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7c, 0xc6, 0xfe, 0xc0, 0xc6, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x1c, 0x36, 0x32, 0x30, 0x7c, 0x30, 0x30, 0x30, 0x78, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x76, 0xcc, 0xcc, 0xcc, 0x7c, 0x0c, 0xcc, 0x78},
{ 0x00, 0x00, 0x00, 0xe0, 0x60, 0x60, 0x6c, 0x76, 0x66, 0x66, 0x66, 0xe6, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x38, 0x18, 0x18, 0x18, 0x18, 0x3c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x06, 0x06, 0x00, 0x0e, 0x06, 0x06, 0x06, 0x06, 0x66, 0x66, 0x3c},
{ 0x00, 0x00, 0x00, 0xe0, 0x60, 0x60, 0x66, 0x6c, 0x78, 0x6c, 0x66, 0xe6, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x38, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x3c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xec, 0xfe, 0xd6, 0xd6, 0xd6, 0xd6, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xdc, 0x66, 0x66, 0x66, 0x66, 0x66, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7c, 0xc6, 0xc6, 0xc6, 0xc6, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xdc, 0x66, 0x66, 0x66, 0x7c, 0x60, 0x60, 0xf0},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x76, 0xcc, 0xcc, 0xcc, 0x7c, 0x0c, 0x0c, 0x1e},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xdc, 0x76, 0x66, 0x60, 0x60, 0xf0, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7c, 0xc6, 0x70, 0x1c, 0xc6, 0x7c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x10, 0x30, 0x30, 0xfc, 0x30, 0x30, 0x30, 0x36, 0x1c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xcc, 0xcc, 0xcc, 0xcc, 0xcc, 0x76, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc6, 0xc6, 0xc6, 0x6c, 0x38, 0x10, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc6, 0xc6, 0xd6, 0xd6, 0xfe, 0x6c, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc6, 0x6c, 0x38, 0x38, 0x6c, 0xc6, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc6, 0xc6, 0xc6, 0xc6, 0x7e, 0x06, 0x0c, 0x78},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfe, 0xcc, 0x18, 0x30, 0x66, 0xfe, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x0e, 0x18, 0x18, 0x18, 0x70, 0x18, 0x18, 0x18, 0x0e, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x70, 0x18, 0x18, 0x18, 0x0e, 0x18, 0x18, 0x18, 0x70, 0x00, 0x00},
{ 0x00, 0x76, 0xdc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x38, 0x6c, 0xc6, 0xc6, 0xfe, 0x00, 0x00, 0x00},
};
#endif

//unsigned int cursorX=0;  //cursor X coordinate
//unsigned int cursorY=0;  //cursor Y coordinate
unsigned int TEXT_cursorX=0;  //Text cursor X coordinate
unsigned int TEXT_cursorY=0;  //Text cursor Y coordinate

//char ls020isbusy=0, ls020isready=0;
char ls020isready=0;

//!!! LCD activatoion tables as it is required by manufacturer!!!
flash char L020_INIT0[]={0xFD, 0xFD, 0xFD, 0xFD};


//First init table for L020 display 20bytes
flash char L020_INIT1[]={ 0xEF, 0x00, 0xEE, 0x04, 0x1B, 0x04, 0xFE, 0xFE, 
                    0xFE, 0xFE, 0xEF, 0x90, 0x4A, 0x04, 0x7F, 0x3F, 0xEE, 0x04, 0x43, 0x06};

//Second init table for L020 display 40 bytes
flash char L020_INIT2[]={ 0xEF, 0x90,  0x09, 0x83,  0x08, 0x00,  0x0B, 0xAF,  0x0A, 0x00,  0x05, 0x00,  0x06, 0x00,
                    0x07, 0x00,  0xEF, 0x00,  0xEE, 0x0C,  0xEF, 0x90,  0x00, 0x80,  0xEF, 0xB0,  0x49, 0x02, 
                    0xEF, 0x00,  0x7F, 0x01,  0xE1, 0x81,  0xE2, 0x02,  0xE2, 0x76,  0xE1, 0x83};

//The last, third init table for L020 display 2 bytes
flash char L020_INIT3[]={ 0x80, 0x01};


#ifdef SOFT_SPI
void SoftSPI_init(void)
{
  LCD_RESET=0 ; //display reset
  DLCD_RESET=1; //set reset as output

  DLCD_MISO=0; //MISO is always input in SPI

  DLCD_MOSI=1; //MOSI must be set as output for SPI

  LCD_CS=1 ; //deselect display
  DLCD_CS=1; //SS must be output for Master mode to work

  LCD_SCK=0; //set SCK lo
  DLCD_SCK=1; //set SCK as output

  LCD_MISO=0;

  LCD_RS=1; //start with RS=high
  DLCD_RS=1; //RS signal is output
  delay_ms(1); // Wait for good reset
  LCD_RESET=1; //release reset
}

#endif  
#ifdef HARD_SPI

void HardSPI_init(void)
{
  LCD_RESET=0 ; //display reset
  DLCD_RESET=1; //set reset as output
  
  DLCD_SS=1; //SS must be output for Master mode to work                          
  
  DLCD_MISO=0; //MISO is always input in SPI
  LCD_MISO=1; //pull up the miso                       
  
  DLCD_MOSI=1; //MOSI must be set as output for SPI

  LCD_CS=1 ; //deselect display
  DLCD_CS=1; //SS must be output for Master mode to work

  LCD_SCK=0; //set SCK lo
  DLCD_SCK=1; //set SCK as output

  LCD_MISO=0;

  LCD_RS=1; //start with RS=high
  DLCD_RS=1; //RS signal is output
  delay_ms(5); // Wait for good reset
  LCD_RESET=1 ; //release reset  
  delay_ms(45);
}
#endif 



void SPI_write(char SPI_Data_Byte, char SPI_data_mode, char CS_enable)
{

#ifdef SOFT_SPI
//SPI_data_mode - sets Command (value 1) or Data (0) 
// Counter used to clock out the data is _General_var
  if (CS_enable) LCD_CS = 1;                                 // Make sure we start with active-low CS high
  LCD_SCK = 0;                                // and CK low
  if (SPI_data_mode)                                // Here we check for the correct transfer mode
    LCD_RS=1;
   else 
    LCD_RS=0;
  if (CS_enable) LCD_CS = 0;                                 // Set active-low CS low to start the SPI cycle 

  for (_General_var = 0; _General_var < 8; _General_var++)         // Prepare to clock out the Address byte
  {
    if (SPI_Data_Byte & 0x80)                          // Check for a 1
      LCD_MOSI = 1;                           // and set the MOSI line appropriately
    else
	 LCD_MOSI = 0; 
     LCD_SCK = 1;                              // Toggle the clock line
     LCD_SCK = 0;
    SPI_Data_Byte <<= 1;                               // Rotate to get the next bit
  }                                                    // and loop back to send the next bit
  if (CS_enable)  LCD_CS = 1;                                 // Finish the data transfer in a correct way
  LCD_MOSI = 0;
#endif  
#ifdef HARD_SPI
 if (CS_enable)  LCD_CS = 1;                                 // Make sure we start with active-low CS high
  if (SPI_data_mode)                                // Here we check for the correct transfer mode
    LCD_RS=1;
   else 
    LCD_RS=0;
  if (CS_enable)  LCD_CS = 0;                                 // Set active-low CS low to start the SPI cycle 
    /* Start transmission */
  spi(SPI_Data_Byte);
  if (CS_enable)  LCD_CS = 1;                                 // Finish the data transfer in a correct way
#endif  


}

void lcd_wrcmd16(unsigned int dat)
{ 
  SPI_write(dat>>8,1,1);
  SPI_write(dat,1,1);
}

void lcd_wrdat16(unsigned int dat)
{ 
  SPI_write(dat>>8,0,1);
  SPI_write(dat,0,1);
}


void lcd_clear(void)
{   
 unsigned int _General_varcls=46464;   
 #asm("cli") 
 // ls020isbusy=1;
  lcd_wrcmd16(0xEF90);
  lcd_wrcmd16(0x0500);
  lcd_wrcmd16(0x0600);
  lcd_wrcmd16(0x0700);
	//now 132*176*2 bytes are to be transmitted
  while (_General_varcls--)
      SPI_write(0xFF,0,1);
  //ls020isbusy=0;     
 #asm("sei")
}


void lcd_fill(unsigned int ls020_fill_color)
{   
    unsigned int _General_varfill=23232;
  #asm("cli") 
  //  ls020isbusy=1;
    lcd_wrcmd16(0xEF90);
    lcd_wrcmd16(0x0500);   
    lcd_wrcmd16(0x0800);
    lcd_wrcmd16(0x0983);
    lcd_wrcmd16(0x0A00);
    lcd_wrcmd16(0x0BAF);   
   
    while (_General_varfill--)
      lcd_wrdat16(ls020_fill_color);   
  #asm("sei")
// ls020isbusy=0;    
}

void lcd_gotoxy(unsigned char lcd_text_x, unsigned char lcd_text_y)
{

  TEXT_cursorX=lcd_text_x;  //Text cursor X coordinate
  TEXT_cursorY=lcd_text_y;  //Text cursor Y coordinate
}



void lcd_reset_cursor(void)
{
 TEXT_cursorY=0; TEXT_cursorX=0;
}




void lcd_putpixel(unsigned int ls020x, unsigned int ls020y)
{
   //0xEF90, 0x0504, 0x08X1, 0x09X2, 0x0AY1, 0x0BY2     
   // ls020isbusy=1;   
     #asm("cli")     
   //     .
 #ifdef LCD_ROT             
    lcd_wrcmd16(0xEF90);
    lcd_wrcmd16(0x0504);   
    lcd_wrcmd16(0x0800+ls020y);
    lcd_wrcmd16(0x0900+ls020y);
    lcd_wrcmd16(0x0A00+ls020x);
    lcd_wrcmd16(0x0B00+ls020x);
    lcd_wrdat16(textcolor);
 #endif
 #ifdef LCD_ROT_NOT      
    lcd_wrcmd16(0xEF90);
    lcd_wrcmd16(0x0504);   
    lcd_wrcmd16(0x0800+ls020x);
    lcd_wrcmd16(0x0900+ls020x);
    lcd_wrcmd16(0x0A00+ls020y);
    lcd_wrcmd16(0x0B00+ls020y);
    lcd_wrdat16(textcolor);
  #endif
// ls020isbusy=0;
#asm("sei")
}



void lcd_cputpixel(unsigned int ls020x, unsigned int ls020y, unsigned int ls020color)
{
   //0xEF90, 0x0504, 0x08X1, 0x09X2, 0x0AY1, 0x0BY2     
// ls020isbusy=1;   
     #asm("cli")     
   //     .
 #ifdef LCD_ROT             
    lcd_wrcmd16(0xEF90);
    lcd_wrcmd16(0x0504);   
    lcd_wrcmd16(0x0800+ls020y);
    lcd_wrcmd16(0x0900+ls020y);
    lcd_wrcmd16(0x0A00+ls020x);
    lcd_wrcmd16(0x0B00+ls020x);
    lcd_wrdat16(ls020color);
 #endif
 #ifdef LCD_ROT_NOT      
    lcd_wrcmd16(0xEF90);
    lcd_wrcmd16(0x0504);   
    lcd_wrcmd16(0x0800+ls020x);
    lcd_wrcmd16(0x0900+ls020x);
    lcd_wrcmd16(0x0A00+ls020y);
    lcd_wrcmd16(0x0B00+ls020y);
    lcd_wrdat16(ls020color);
    //lcd_wrdat16(ls020color); 
  #endif
// ls020isbusy=0;
#asm("sei")
}


void lcd_fillrect(unsigned int ls020x1, unsigned int ls020y1, 
                  unsigned int ls020x2, unsigned int ls020y2, unsigned int ls020color)
{
  
  unsigned int ls020deltax=0, ls020deltay=0,_General_varfillrect;   
   #asm("cli")
// ls020isbusy=1;   
  _General_varfillrect=0;               
  //0xEF90, 0x0504, 0x08X1, 0x09X2, 0x0AY1, 0x0BY2
  //We draw one direction only!! Left-Right Down - up  
  if (ls020x1>ls020x2) { _General_varfillrect = ls020x1; ls020x1=ls020x2; ls020x2=_General_varfillrect;} ;
  if (ls020y1>ls020y2) { _General_varfillrect = ls020y1; ls020y1=ls020y2; ls020y2=_General_varfillrect;} ;  
  ls020deltax=ls020x2-ls020x1;                                                                   
  ls020deltay=ls020y2-ls020y1;
  _General_varfillrect=ls020deltax*ls020deltay;     
  lcd_wrcmd16(0xEF90);
  lcd_wrcmd16(0x0504);
  lcd_wrcmd16(0x0800+ls020x1);
  lcd_wrcmd16(0x0900+ls020x1+ls020deltax-1);
  lcd_wrcmd16(0x0A00+ls020y1);
  lcd_wrcmd16(0x0B00+ls020y1+ls020deltay-1);
  while (_General_varfillrect--)
    lcd_wrdat16(ls020color);   
 //ls020isbusy=0;
 #asm("sei")    
}


unsigned int lcd_color16(unsigned long int ls020color24)
{ // 24   16
return ((ls020color24>>8)&0xF800)+((ls020color24>>5)&0x07E0)+((ls020color24>>3)&0x001F);
}



//####################################################################################
void LS020_lcd_init(void)
{ 
 
  unsigned int _General_varinit;
  //init tables counter set to start in _General_var
  #asm("cli")
  LCD_CS=1;    //deselect display
  DLCD_CS=1;
  // Initialise the SPI input output pins
  #ifdef SOFT_SPI
     SoftSPI_init();
  #endif  
  #ifdef HARD_SPI
     HardSPI_init();
  #endif  
  // Zero 4 bytes table shoot
  for (_General_varinit=0; _General_varinit<4; _General_varinit++)
  {
    SPI_write(L020_INIT0[_General_varinit],1,1);
  }
  delay_ms(6);   
  for (_General_varinit=0; _General_varinit<20; _General_varinit++)
  {
    SPI_write(L020_INIT1[_General_varinit],1,1);
  }          
  delay_ms(6);
   // Second table shoot
  for (_General_varinit=0; _General_varinit<40; _General_varinit++)
  {
    SPI_write(L020_INIT2[_General_varinit],1,1);
  }          
  delay_ms(49); 
   // Third table shoot
  for (_General_varinit=0; _General_varinit<2; _General_varinit++)
  {
    SPI_write(L020_INIT3[_General_varinit],1,1);
  }   
  delay_ms(1);  
  SPI_write(0xEF,1,1);
  SPI_write(0x90,1,1);
  SPI_write(0x00,1,1);
  SPI_write(0x00,1,1);
  lcd_clear();  
  ls020isready=1;  
  #asm("sei")  
}


void lcd_d(unsigned int data)
{ 
  lcd_wrdat16(data);
} 
//   (2 )
void lcd_c(unsigned int command)
{ 
  lcd_wrcmd16(command);
}


void put_char(char x, char y, char c, unsigned int color)
{
  char h,ch,p,mask,tt;
  //char m=1;
 // DISP_READY=0;
  x=DISP_H - CHAR_H - x;
    lcd_c(0xEF90); 
    lcd_c(0x0504);
    lcd_c(0x0800+y);
    lcd_c(0x0A00+x);
    lcd_c(0x0900+y+CHAR_H-1);
    lcd_c(0x0B00+x+CHAR_W-1);
  for (h=CHAR_H; h>0; h--) // every column of the character
  {
      for(tt=0;tt<1;tt++)
        {  
        ch=ascii_tab[c][CHAR_H-h];
        mask=0x01;
        for (p=0; p<CHAR_W; p++)  // write the pixels
          {
          if (ch&mask)
            {
            lcd_d(color);
            //if(m==2) lcd_d(color);                 
            }
           else
            {
            lcd_d(backcolor);
            //if(m==2) lcd_d(bgcolor); 
            }
          mask=mask*2;
          }
        }  
  }
    
}

void put_string(unsigned char x, unsigned char y, char *str, unsigned int color)
{
unsigned char  j=0;
  while (j<strlen(str))
   {
   put_char(x+j*CHAR_W,y,str[j],color);   //  0 deg. rotated str[j]
   j++;
   }
} 





void ls020_power_OFF(void)
{

/*0xEF00, 0x7E04
- Pause 60mSec
0xEFB0, 0x5A48, 0xEF00, 0x7F01
- Pause 60mSec
0xEFB0, 0x64FF, 0x6500, 0xEF00, 0x7F01
- Pause 60mSec
0xE262
- Pause 100mSec
0xE202
- Pause 60mSec
0xEFB0, 0xBC02, 0xEF00, 0x7F01
- Pause 20mSec
0xE200
- Pause 10mSec
0x8000, 0xE204
- Pause 20mSec
0xE200
- Pause 20mSec
0xE100
- Pause 20mSec
0xEFB0, 0xBC00, 0xEF00, 0x7F01*/


  LCD_RESET=0 ; //display reset
  DLCD_RESET=1; //set reset as output
  delay_ms(60);
  LCD_RESET=0; //display reset   
  DLCD_RESET=0; //set reset as output
}